@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2008 - 2019, Ilan Schnell
+   Copyright (c) 2008 - 2020, Ilan Schnell
    bitarray is published under the PSF license.
 
    This file is the C part of the bitarray package.
@@ -36,7 +36,7 @@
 typedef long long int idx_t;
 
 /* throughout:  0 = little endian   1 = big endian */
-#define DEFAULT_ENDIAN  1
+static int default_endian = 1;
 
 /* Unlike the normal convention, ob_size is the byte count, not the number
    of elements.  The reason for doing this is that we can use our own
@@ -54,7 +54,9 @@ typedef struct {
 
 static PyTypeObject Bitarraytype;
 
-#define bitarray_Check(obj)  PyObject_TypeCheck(obj, &Bitarraytype)
+#define ENDIAN_STR(ba)  (((ba)->endian) ? "big" : "little")
+
+#define bitarray_Check(obj)  PyObject_TypeCheck((obj), &Bitarraytype)
 
 #define BITS(bytes)  ((idx_t) (bytes) << 3)
 
@@ -290,6 +292,7 @@ setunused(bitarrayobject *self)
         res++;
     }
     assert(res < 8);
+    assert(n == self->nbits + res);
     return res;
 }
 
@@ -425,8 +428,7 @@ bytereverse(bitarrayobject *self)
     }
 }
 
-
-static int bitcount_lookup[256] = {
+static unsigned char bitcount_lookup[256] = {
     0, 1, 1, 2, 1, 2, 2, 3, 1, 2, 2, 3, 2, 3, 3, 4,
     1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5,
     1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5,
@@ -587,21 +589,20 @@ unpack(bitarrayobject *self, char zero, char one)
 static int
 extend_bitarray(bitarrayobject *self, bitarrayobject *other)
 {
-    idx_t n_sum;
-    idx_t n_other_bits;
+    idx_t self_bits, other_bits;
 
     if (other->nbits == 0)
         return 0;
 
-    /* Note that other may be self.  Thus we take the size before we resize,
-       ensuring we only copy the right parts of the array. */
-    n_other_bits = other->nbits;
-    n_sum = self->nbits + other->nbits;
+    /* We have to store the sizes before we resize.
+       Since other may be self, we also need to store other->nbits. */
+    self_bits = self->nbits;
+    other_bits = other->nbits;
 
-    if (resize(self, n_sum) < 0)
+    if (resize(self, self_bits + other_bits) < 0)
         return -1;
 
-    copy_n(self, n_sum - n_other_bits, other, 0, n_other_bits);
+    copy_n(self, self_bits, other, 0, other_bits);
     return 0;
 }
 
@@ -781,17 +782,17 @@ extend_dispatch(bitarrayobject *self, PyObject *obj)
     return ret;
 }
 
-/* --------- helper functions NOT involving bitarrayobjects ------------ */
-
-#define ENDIAN_STR(ba)  (((ba)->endian) ? "big" : "little")
+/* --------- helper functions not involving bitarrayobjects ------------ */
 
 #ifdef IS_PY3K
 #define IS_INDEX(x)  (PyLong_Check(x) || PyIndex_Check(x))
 #define IS_INT_OR_BOOL(x)  (PyBool_Check(x) || PyLong_Check(x))
+#define Py_CSTRING  PyUnicode_FromString
 #else  /* Py 2 */
 #define IS_INDEX(x)  (PyInt_Check(x) || PyLong_Check(x) || PyIndex_Check(x))
 #define IS_INT_OR_BOOL(x)  (PyBool_Check(x) || PyInt_Check(x) || \
                                                PyLong_Check(x))
+#define Py_CSTRING  PyString_FromString
 #endif
 
 /* given an PyLong (which must be 0 or 1), or a PyBool, return 0 or 1,
@@ -1163,11 +1164,7 @@ contents, the bit endianness as a string, the number of unused bits\n\
 static PyObject *
 bitarray_endian(bitarrayobject *self)
 {
-#ifdef IS_PY3K
-    return PyUnicode_FromString(ENDIAN_STR(self));
-#else
-    return PyString_FromString(ENDIAN_STR(self));
-#endif
+    return Py_CSTRING(ENDIAN_STR(self));
 }
 
 PyDoc_STRVAR(endian_doc,
@@ -2320,8 +2317,9 @@ make_tree(PyObject *codedict)
 
 /*
   Traverse tree using the branches corresponding to the bitarray `ba`,
-  starting at *indexp, and return the symbol at the leaf node (or NULL
-  when the end of the bitarray has been reached).
+  starting at *indexp.  Return the symbol at the leaf node, or NULL
+  when the end of the bitarray has been reached, or on error (in which
+  case the appropriate PyErr_SetString is set.
 */
 static PyObject *
 traverse_tree(binode *tree, bitarrayobject *ba, idx_t *indexp)
@@ -2739,6 +2737,27 @@ bitarray_methods[] = {
 };
 
 
+/* Given a string, return an integer representing the endianness.
+   If the string is invalid, set a Python exception and return -1. */
+static int
+endian_from_string(const char* string)
+{
+    assert(default_endian == 0 || default_endian == 1);
+
+    if (string == NULL)
+        return default_endian;
+
+    if (strcmp(string, "little") == 0)
+        return 0;
+
+    if (strcmp(string, "big") == 0)
+        return 1;
+
+    PyErr_SetString(PyExc_ValueError, "endian must be 'little' or 'big'");
+    return -1;
+}
+
+
 static PyObject *
 bitarray_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 {
@@ -2746,26 +2765,15 @@ bitarray_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
     PyObject *initial = NULL;
     char *endian_str = NULL;
     int endian;
-    static char *kwlist[] = {"initial", "endian", NULL};
+    static char *kwlist[] = {"", "endian", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds,
                         "|Os:bitarray", kwlist, &initial, &endian_str))
         return NULL;
 
-    if (endian_str == NULL) {
-        endian = DEFAULT_ENDIAN;  /* use default value */
-    }
-    else if (strcmp(endian_str, "little") == 0) {
-        endian = 0;
-    }
-    else if (strcmp(endian_str, "big") == 0) {
-        endian = 1;
-    }
-    else {
-        PyErr_SetString(PyExc_ValueError,
-                        "endian must be 'little' or 'big'");
+    endian = endian_from_string(endian_str);
+    if (endian < 0)
         return NULL;
-    }
 
     /* no arg or None */
     if (initial == NULL || initial == Py_None)
@@ -3095,6 +3103,12 @@ static PyBufferProcs bitarray_as_buffer = {
 
 #endif  /* WITH_BUFFER */
 
+PyDoc_STRVAR(bitarraytype_doc,
+"_bitarray(initializer=0, /, endian='big') -> bitarray\n\
+\n\
+Return a new bitarray object.\n\
+");
+
 /************************** Bitarray Type *******************************/
 
 static PyTypeObject Bitarraytype = {
@@ -3132,7 +3146,7 @@ static PyTypeObject Bitarraytype = {
     | Py_TPFLAGS_HAVE_NEWBUFFER
 #endif
     ,                                         /* tp_flags */
-    0,                                        /* tp_doc */
+    bitarraytype_doc,                         /* tp_doc */
     0,                                        /* tp_traverse */
     0,                                        /* tp_clear */
     richcompare,                              /* tp_richcompare */
@@ -3221,6 +3235,44 @@ PyDoc_STRVAR(bits2bytes_doc,
 Return the number of bytes necessary to store n bits.");
 
 
+static PyObject *
+get_default_endian(PyObject *self)
+{
+    return Py_CSTRING(default_endian ? "big" : "little");
+}
+
+PyDoc_STRVAR(get_default_endian_doc,
+"get_default_endian() -> string\n\
+\n\
+Return the default endianness for new bitarray objects being created.\n\
+Under normal circumstances, the return value is `big`.");
+
+
+static PyObject *
+set_default_endian(PyObject *self, PyObject *args)
+{
+    char *endian_str;
+    int tmp;
+
+    if (!PyArg_ParseTuple(args, "s:_set_default_endian", &endian_str))
+        return NULL;
+
+    /* As endian_from_string might return -1, we have to store its value
+       in a temporary variable before setting default_endian. */
+    tmp = endian_from_string(endian_str);
+    if (tmp < 0)
+        return NULL;
+    default_endian = tmp;
+
+    Py_RETURN_NONE;
+}
+
+PyDoc_STRVAR(set_default_endian_doc,
+"_set_default_endian(endian, /)\n\
+\n\
+Set the default bit endianness for new bitarray objects being created.");
+
+
 static PyObject *
 sysinfo(void)
 {
@@ -3250,6 +3302,10 @@ tuple(sizeof(void *),\n\
 static PyMethodDef module_functions[] = {
     {"bitdiff",    (PyCFunction) bitdiff,    METH_VARARGS, bitdiff_doc   },
     {"bits2bytes", (PyCFunction) bits2bytes, METH_O,       bits2bytes_doc},
+    {"get_default_endian", (PyCFunction) get_default_endian, METH_NOARGS,
+                                                   get_default_endian_doc},
+    {"_set_default_endian", (PyCFunction) set_default_endian, METH_VARARGS,
+                                                   set_default_endian_doc},
     {"_sysinfo",   (PyCFunction) sysinfo,    METH_NOARGS,  sysinfo_doc   },
     {NULL,         NULL}  /* sentinel */
 };
