@@ -29,9 +29,9 @@ else:
     from cStringIO import StringIO
 
 
-from bitarray import (bitarray, frozenbitarray,
-                      bitdiff, bits2bytes, _sysinfo, __version__)
-
+from bitarray import (bitarray, frozenbitarray, bitdiff, bits2bytes,
+                      get_default_endian, _set_default_endian,
+                      _sysinfo, __version__)
 
 tests = []
 
@@ -39,7 +39,7 @@ tests = []
 class Util(object):
 
     @staticmethod
-    def randombitarrays(start=1):
+    def randombitarrays(start=0):
         for n in list(range(start, 25)) + [randint(1000, 2000)]:
             a = bitarray(endian=['little', 'big'][randint(0, 1)])
             a.frombytes(os.urandom(bits2bytes(n)))
@@ -58,6 +58,12 @@ class Util(object):
         else:
             return randint(-length-5, length+5)
 
+    @staticmethod
+    def other_endian(endian):
+        t = {'little': 'big',
+             'big': 'little'}
+        return t[endian]
+
     @staticmethod
     def slicelen(s, length):
         assert isinstance(s, slice)
@@ -92,6 +98,28 @@ class Util(object):
 
 class TestsModuleFunctions(unittest.TestCase, Util):
 
+    def test_default_endian(self):
+        self.assertRaises(TypeError, _set_default_endian, 0)
+        self.assertRaises(TypeError, _set_default_endian, 'little', 0)
+        self.assertRaises(ValueError, _set_default_endian, 'foo')
+        for default_endian in 'big', 'little', u'big', u'little':
+            _set_default_endian(default_endian)
+            a = bitarray()
+            self.assertEqual(a.endian(), default_endian)
+            a = bitarray('10111')
+            self.assertEqual(a.endian(), default_endian)
+
+            for endian in 'big', 'little':
+                a = bitarray(endian=endian)
+                self.assertEqual(a.endian(), endian)
+
+            # test get_default_endian()
+            self.assertEqual(get_default_endian(), default_endian)
+            # make sure that calling _set_default_endian wrong does not
+            # change the default endianness
+            self.assertRaises(ValueError, _set_default_endian, 'foobar')
+            self.assertEqual(get_default_endian(), default_endian)
+
     def test_bitdiff(self):
         a = bitarray('0011')
         b = bitarray('0101')
@@ -148,34 +176,23 @@ class CreateObjectTests(unittest.TestCase, Util):
         self.assertEqual(a.tolist(), [])
         self.check_obj(a)
 
-    def test_endian1(self):
+    def test_endian(self):
         a = bitarray(endian='little')
-        a.fromstring('A')
+        a.frombytes(b'ABC')
         self.assertEqual(a.endian(), 'little')
         self.assertIsInstance(a.endian(), str)
         self.check_obj(a)
 
         b = bitarray(endian='big')
-        b.fromstring('A')
+        b.frombytes(b'ABC')
         self.assertEqual(b.endian(), 'big')
         self.assertIsInstance(a.endian(), str)
         self.check_obj(b)
 
-        self.assertEqual(a.tostring(), b.tostring())
-
-    def test_endian2(self):
-        a = bitarray(endian='little')
-        a.fromstring(' ')
-        self.assertEqual(a.endian(), 'little')
-        self.check_obj(a)
-
-        b = bitarray(endian='big')
-        b.fromstring(' ')
-        self.assertEqual(b.endian(), 'big')
-        self.check_obj(b)
-
-        self.assertEqual(a.tostring(), b.tostring())
+        self.assertNotEqual(a, b)
+        self.assertEqual(a.tobytes(), b.tobytes())
 
+    def test_endian_wrong(self):
         self.assertRaises(TypeError, bitarray.__new__, bitarray, endian=0)
         self.assertRaises(ValueError, bitarray.__new__, bitarray, endian='')
         self.assertRaises(ValueError, bitarray.__new__,
@@ -255,7 +272,7 @@ class CreateObjectTests(unittest.TestCase, Util):
 
         self.assertRaises(ValueError, bitarray.__new__, bitarray, '01012100')
 
-    def test_rawbytes(self): # this representation is used for pickling
+    def test_rawbytes(self):  # this representation is used for pickling
         for s, r in [(b'\x00', ''), (b'\x07\xff', '1'), (b'\x03\xff', '11111'),
                      (b'\x01\x87\xda', '10000111' '1101101')]:
             self.assertEqual(bitarray(s, endian='big'),
@@ -265,28 +282,56 @@ class CreateObjectTests(unittest.TestCase, Util):
             self.assertRaises(ValueError, bitarray.__new__,
                               bitarray, bytes(bytearray([i])))
 
-    def test_bitarray(self):
+    def test_bitarray_simple(self):
         for n in range(10):
             a = bitarray(n)
             b = bitarray(a)
             self.assertFalse(a is b)
             self.assertEQUAL(a, b)
 
-        for end in ('little', 'big'):
-            a = bitarray(endian=end)
-            c = bitarray(a)
-            self.assertEqual(c.endian(), end)
-            c = bitarray(a, endian='little')
-            self.assertEqual(c.endian(), 'little')
-            c = bitarray(a, endian='big')
-            self.assertEqual(c.endian(), 'big')
+    def test_bitarray_endian(self):
+        # Test creating a new bitarray with different endianness from an
+        # existing bitarray.
+        for endian in 'little', 'big':
+            a = bitarray(endian=endian)
+            b = bitarray(a)
+            self.assertFalse(a is b)
+            self.assertEQUAL(a, b)
 
+            endian2 = self.other_endian(endian)
+            c = bitarray(a, endian2)
+            self.assertEqual(c.endian(), endian2)
+            self.assertEqual(a, c)  # but only because they are empty
+
+            # Even though the byte representation will be the same,
+            # the bitarrays are not equal.
+            a = bitarray('11001000' '11110000', endian)
+            self.assertEqual(len(a) % 8, 0)
+            c = bitarray(a, endian2)
+            # This is only equal because the size of the bitarray is a
+            # multiple of 8, and unused bits are not set (which changes
+            # the byte representation).
+            self.assertEqual(a.tobytes(), c.tobytes())
+            self.assertNotEqual(a.endian(), c.endian())
+            self.assertNotEqual(a, c)
+
+    def test_bitarray_endianness(self):
+        a = bitarray('11100001', endian='little')
+        b = bitarray(a, endian='big')
+        self.assertNotEqual(a, b)
+        self.assertEqual(a.tobytes(), b.tobytes())
+
+        b.bytereverse()
+        self.assertEqual(a, b)
+        self.assertNotEqual(a.tobytes(), b.tobytes())
+
+        c = bitarray('11100001', endian='big')
+        self.assertEqual(a, c)
 
     def test_None(self):
         self.assertEQUAL(bitarray(), bitarray(0))
         self.assertEQUAL(bitarray(), bitarray(None))
 
-
     def test_WrongArgs(self):
         self.assertRaises(TypeError, bitarray.__new__, bitarray, 'A', 42, 69)
 
@@ -297,6 +342,9 @@ class CreateObjectTests(unittest.TestCase, Util):
         self.assertRaises(TypeError, bitarray.__new__, bitarray, 4+3j)
 
         self.assertRaises(TypeError, bitarray.__new__, bitarray, '', 0, 42)
+        # test second (endian) argument
+        self.assertRaises(TypeError, bitarray.__new__, bitarray, 0, None)
+        self.assertRaises(TypeError, bitarray.__new__, bitarray, 0, 0)
         self.assertRaises(ValueError, bitarray.__new__, bitarray, 0, 'foo')
 
 
@@ -514,7 +562,7 @@ class SliceTests(unittest.TestCase, Util):
         a = bitarray('1111')
         a[1:3] = bitarray('0000')
         self.assertEqual(a, bitarray('100001'))
-        a[:] = bitarray('010') # replace all values
+        a[:] = bitarray('010')  # replace all values
         self.assertEqual(a, bitarray('010'))
 
     def test_setslice_to_bool(self):
@@ -547,7 +595,7 @@ class SliceTests(unittest.TestCase, Util):
         self.assertRaises(ValueError, a.__setitem__, slice(None, None, 2), 3)
         self.assertRaises(ValueError, a.__setitem__, slice(None, 2, None), -1)
 
-    def test_sieve(self): # Sieve of Eratosthenes
+    def test_sieve(self):  # Sieve of Eratosthenes
         a = bitarray(50)
         a.setall(1)
         for i in range(2, 8):
@@ -667,7 +715,7 @@ class MiscTests(unittest.TestCase, Util):
             def __getitem__(self, i):
                 return bitarray.__getitem__(self, i - self.offset)
 
-        for a in self.randombitarrays(start=1):
+        for a in self.randombitarrays(start=0):
             b = ExaggeratingBitarray(a, 1234)
             for i in range(len(a)):
                 self.assertEqual(a[i], b[i+1234])
@@ -986,7 +1034,7 @@ class BitwiseTests(unittest.TestCase, Util):
         self.assertEQUAL(a & b, bitarray('10001'))
 
         b = bitarray('1001')
-        self.assertRaises(ValueError, a.__and__, b) # not same length
+        self.assertRaises(ValueError, a.__and__, b)  # not same length
 
         self.assertRaises(TypeError, a.__and__, 42)
 
@@ -1274,9 +1322,9 @@ class MethodTests(unittest.TestCase, Util):
             b.append(1)
             self.assertTrue(a is b)
             self.check_obj(b)
-            self.assertEQUAL(b, bitarray(aa+[1], endian=a.endian()))
+            self.assertEQUAL(b, bitarray(aa + [1], endian=a.endian()))
             b.append('')
-            self.assertEQUAL(b, bitarray(aa+[1, 0], endian=a.endian()))
+            self.assertEQUAL(b, bitarray(aa + [1, 0], endian=a.endian()))
 
 
     def test_insert(self):
@@ -1384,17 +1432,17 @@ class MethodTests(unittest.TestCase, Util):
             i = randint(0, 1)
             a.setall(i)
             for unused in range(randint(1, 4)):
-                a[randint(0, n-1)] = 1-i
+                a[randint(0, n - 1)] = 1 - i
             aa = a.tolist()
             for unused in range(100):
-                start = randint(-50, n+50)
-                stop = randint(-50, n+50)
+                start = randint(-50, n + 50)
+                stop = randint(-50, n + 50)
                 try:
-                    res1 = a.index(1-i, start, stop)
+                    res1 = a.index(1 - i, start, stop)
                 except ValueError:
                     res1 = None
                 try:
-                    res2 = aa.index(1-i, start, stop)
+                    res2 = aa.index(1 - i, start, stop)
                 except ValueError:
                     res2 = None
                 self.assertEqual(res1, res2)
@@ -1448,8 +1496,8 @@ class MethodTests(unittest.TestCase, Util):
     def test_count6(self):
         for a in self.randombitarrays():
             s = a.to01()
-            i = randint(-3, len(a)+1)
-            j = randint(-3, len(a)+1)
+            i = randint(-3, len(a) + 1)
+            j = randint(-3, len(a) + 1)
             self.assertEqual(a.count(1, i, j), s[i:j].count('1'))
             self.assertEqual(a.count(0, i, j), s[i:j].count('0'))
 
@@ -1499,7 +1547,7 @@ class MethodTests(unittest.TestCase, Util):
     def test_search3(self):
         a = bitarray('10010101110011111001011')
         for s, res in [('011', [6, 11, 20]),
-                       ('111', [7, 12, 13, 14]), # note the overlap
+                       ('111', [7, 12, 13, 14]),  # note the overlap
                        ('1011', [5, 19]),
                        ('100', [0, 9, 16])]:
             b = bitarray(s)
@@ -1665,7 +1713,7 @@ class MethodTests(unittest.TestCase, Util):
 
         for a in self.randombitarrays():
             self.assertRaises(IndexError, a.pop, len(a))
-            self.assertRaises(IndexError, a.pop, -len(a)-1)
+            self.assertRaises(IndexError, a.pop, -len(a) - 1)
             if len(a) == 0:
                 continue
             aa = a.tolist()
@@ -1725,7 +1773,7 @@ tests.append(MethodTests)
 
 # ---------------------------------------------------------------------------
 
-class StringTests(unittest.TestCase, Util):
+class BytesTests(unittest.TestCase, Util):
 
     def randombytes(self):
         for n in range(1, 20):
@@ -1831,7 +1879,7 @@ class StringTests(unittest.TestCase, Util):
         self.assertRaises(TypeError, a.pack, bitarray())
 
 
-tests.append(StringTests)
+tests.append(BytesTests)
 
 # ---------------------------------------------------------------------------
 
@@ -1877,7 +1925,7 @@ class FileTests(unittest.TestCase, Util):
     def test_fromfile_wrong_args(self):
         b = bitarray()
         self.assertRaises(TypeError, b.fromfile)
-        self.assertRaises(TypeError, b.fromfile, StringIO()) # file not open
+        self.assertRaises(TypeError, b.fromfile, StringIO())  # file not open
         self.assertRaises(TypeError, b.fromfile, 42)
         self.assertRaises(TypeError, b.fromfile, 'bar')
 
@@ -1932,33 +1980,33 @@ class FileTests(unittest.TestCase, Util):
 
     def test_fromfile_n(self):
         a = bitarray()
-        a.fromstring('ABCDEFGHIJ')
+        a.frombytes(b'ABCDEFGHIJ')
         with open(self.tmpfname, 'wb') as fo:
             a.tofile(fo)
 
         b = bitarray()
         with open(self.tmpfname, 'rb') as f:
-            b.fromfile(f, 0);     self.assertEqual(b.tostring(), '')
-            b.fromfile(f, 1);     self.assertEqual(b.tostring(), 'A')
+            b.fromfile(f, 0);     self.assertEqual(b.tobytes(), b'')
+            b.fromfile(f, 1);     self.assertEqual(b.tobytes(), b'A')
             f.read(1)
             b = bitarray()
-            b.fromfile(f, 2);     self.assertEqual(b.tostring(), 'CD')
-            b.fromfile(f, 1);     self.assertEqual(b.tostring(), 'CDE')
-            b.fromfile(f, 0);     self.assertEqual(b.tostring(), 'CDE')
-            b.fromfile(f);        self.assertEqual(b.tostring(), 'CDEFGHIJ')
-            b.fromfile(f);        self.assertEqual(b.tostring(), 'CDEFGHIJ')
+            b.fromfile(f, 2);     self.assertEqual(b.tobytes(), b'CD')
+            b.fromfile(f, 1);     self.assertEqual(b.tobytes(), b'CDE')
+            b.fromfile(f, 0);     self.assertEqual(b.tobytes(), b'CDE')
+            b.fromfile(f);        self.assertEqual(b.tobytes(), b'CDEFGHIJ')
+            b.fromfile(f);        self.assertEqual(b.tobytes(), b'CDEFGHIJ')
 
         b = bitarray()
         with open(self.tmpfname, 'rb') as f:
-            f.read(1);
+            f.read(1)
             self.assertRaises(EOFError, b.fromfile, f, 10)
 
-        self.assertEqual(b.tostring(), 'BCDEFGHIJ')
+        self.assertEqual(b.tobytes(), b'BCDEFGHIJ')
 
         b = bitarray()
         with open(self.tmpfname, 'rb') as f:
-            b.fromfile(f);
-            self.assertEqual(b.tostring(), 'ABCDEFGHIJ')
+            b.fromfile(f)
+            self.assertEqual(b.tobytes(), b'ABCDEFGHIJ')
             self.assertRaises(EOFError, b.fromfile, f, 1)
 
 
@@ -1986,7 +2034,7 @@ class FileTests(unittest.TestCase, Util):
             self.assertEqual(len(s), a.buffer_info()[1])
 
         for n in range(3):
-            a.fromstring(n * 'A')
+            a.frombytes(n * b'A')
             self.assertRaises(TypeError, a.tofile)
             self.assertRaises(TypeError, a.tofile, StringIO())
 
@@ -2288,7 +2336,7 @@ class BufferInterfaceTests(unittest.TestCase):
         a[4003] = 0
         self.assertEqual(a[3999:4009], bitarray('0111011110'))
         v[301:304] = b'ABC'
-        self.assertEqual(a[300 *8 : 305 * 8].tobytes(), b'\x00ABC\x00')
+        self.assertEqual(a[300 * 8 : 305 * 8].tobytes(), b'\x00ABC\x00')
 
 if sys.version_info[:2] >= (2, 7):
     tests.append(BufferInterfaceTests)
@@ -2353,7 +2401,7 @@ class TestsFrozenbitarray(unittest.TestCase, Util):
         self.assertEqual(d[frozenbitarray('01')], 123)
         self.assertEqual(d[frozenbitarray(b)], 345)
 
-    def test_dictkey2(self): # taken sllightly modified from issue #74
+    def test_dictkey2(self):  # taken slightly modified from issue #74
         a1 = frozenbitarray([True, False])
         a2 = frozenbitarray([False, False])
         dct = {a1: "one", a2: "two"}
@@ -2378,8 +2426,9 @@ def run(verbosity=1, repeat=1):
 
     print('bitarray is installed in: %s' % os.path.dirname(__file__))
     print('bitarray version: %s' % __version__)
-    print('Python version: %s' % sys.version)
-
+    print('sys.version: %s' % sys.version)
+    print('sys.prefix: %s' % sys.prefix)
+    print('pointer size: %d bit' % (8 * _sysinfo()[0]))
     suite = unittest.TestSuite()
     for cls in tests:
         for _ in range(repeat):
